@@ -55,31 +55,19 @@
 /* Global Variables */
 
 extern GtkWidget *gm;
-extern GnomeMeeting *MyApp;
+
+int GDKVideoOutputDevice::devices_nbr = 0;
 
 
 /* The Methods */
-GDKVideoOutputDevice::GDKVideoOutputDevice(GmWindow *w)
-{
-  gw = w;
-
-  /* Used to distinguish between input and output device. */
-  device_id = REMOTE; 
-
-  
-#ifdef HAS_SDL
-  screen = NULL;
-  overlay = NULL;
-#endif
-}
-
-
 GDKVideoOutputDevice::GDKVideoOutputDevice(int idno, GmWindow *w)
 {
   gw = w;
 
   /* Used to distinguish between input and output device. */
   device_id = idno; 
+
+  devices_nbr++;
   
 #ifdef HAS_SDL
   screen = NULL;
@@ -106,10 +94,10 @@ GDKVideoOutputDevice::~GDKVideoOutputDevice()
   }
 #endif
 
-
+  devices_nbr--;
+  
   /* Hide the 2 popup windows */
   gnomemeeting_threads_enter ();
-
   gtk_widget_hide_all (GTK_WIDGET (gw->local_video_window));
   gtk_widget_hide_all (GTK_WIDGET (gw->remote_video_window));
   gnomemeeting_threads_leave ();
@@ -161,7 +149,7 @@ BOOL GDKVideoOutputDevice::Redraw ()
   /* Updates the zoom value */
   zoom = 
     gconf_client_get_float (client, 
-			    "/apps/gnomemeeting/video_display/zoom_factor", 
+			    VIDEO_DISPLAY_KEY "zoom_factor", 
 			    NULL);
   if (zoom != 0.5 && zoom != 1 && zoom != 2)
     zoom = 1.0;
@@ -169,7 +157,7 @@ BOOL GDKVideoOutputDevice::Redraw ()
 #ifdef HAS_SDL 
   fullscreen =
     gconf_client_get_bool (client, 
-			  "/apps/gnomemeeting/video_display/fullscreen", 0);
+			  VIDEO_DISPLAY_KEY "fullscreen", 0);
 #endif
 
 
@@ -177,7 +165,7 @@ BOOL GDKVideoOutputDevice::Redraw ()
   
   /* What do we display: what gconf tells us except if we are not in 
      a call, or if it is not a valid choice */
-  if (MyApp->Endpoint ()->GetCallingState () != 2) {
+  if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () != 2) {
 
     display = 0;
   }
@@ -185,17 +173,18 @@ BOOL GDKVideoOutputDevice::Redraw ()
 
     display = 
       gconf_client_get_int (client, 
-			    "/apps/gnomemeeting/video_display/video_view", 
+			    VIDEO_DISPLAY_KEY "video_view", 
 			    NULL);
 
-    if (MyApp->Endpoint ()->GetVideoChannelsNumber () <= 1)
+    if (devices_nbr <= 1)
       if (device_id == REMOTE)
 	display = REMOTE_VIDEO;
       else if (device_id == LOCAL)
 	display = LOCAL_VIDEO;
   }
 
-  gnomemeeting_video_submenu_select (display);
+  gtk_radio_menu_select_with_id (gw->main_menu, "local_video", display);
+  gtk_radio_menu_select_with_id (gw->video_popup_menu, "local_video", display);
 
   /* Show or hide the different windows if needed */
   if (display == BOTH) { /* display == BOTH */
@@ -213,22 +202,13 @@ BOOL GDKVideoOutputDevice::Redraw ()
     if (!GTK_WIDGET_VISIBLE (gw->remote_video_window))
       gtk_widget_show_all (GTK_WIDGET (gw->remote_video_window));
 
-    if (device_id == REMOTE) {
-
-      gtk_window_get_size (GTK_WINDOW (gw->remote_video_window), 
-			   &zoomed_width, &zoomed_height);
-      zoomed_width = zoomed_width - GM_FRAME_SIZE;
-      zoomed_height = zoomed_height - GM_FRAME_SIZE;
+    if (device_id == REMOTE) 
       image = gw->remote_video_image;
-    }
-    else {
-
-      gtk_window_get_size (GTK_WINDOW (gw->local_video_window), 
-			   &zoomed_width, &zoomed_height);
-      zoomed_width = zoomed_width - GM_FRAME_SIZE;
-      zoomed_height = zoomed_height - GM_FRAME_SIZE;
+    else 
       image = gw->local_video_image;
-    } 
+
+    gtk_widget_get_size_request (GTK_WIDGET (image), 
+				 &zoomed_width, &zoomed_height);
   }
   else {
 
@@ -246,11 +226,8 @@ BOOL GDKVideoOutputDevice::Redraw ()
       if (device_id == LOCAL) {
 
 	image = gw->local_video_image;
-
-	gtk_window_get_size (GTK_WINDOW (gw->local_video_window), 
-			     &zoomed_width, &zoomed_height);
-	zoomed_width = zoomed_width - GM_FRAME_SIZE;
-	zoomed_height = zoomed_height - GM_FRAME_SIZE;
+	gtk_widget_get_size_request (GTK_WIDGET (image), 
+				     &zoomed_width, &zoomed_height);
       }
 
       if (!GTK_WIDGET_VISIBLE (gw->local_video_window))
@@ -297,11 +274,11 @@ BOOL GDKVideoOutputDevice::Redraw ()
       
     zoomed_width = 
       gconf_client_get_int (client, 
-			    "/apps/gnomemeeting/video_display/fullscreen_width", 
+			    VIDEO_DISPLAY_KEY "fullscreen_width", 
 			    NULL);
     zoomed_height = 
       gconf_client_get_int (client, 
-			    "/apps/gnomemeeting/video_display/fullscreen_height", 
+			    VIDEO_DISPLAY_KEY "fullscreen_height", 
 			    NULL);
     }
 #endif
@@ -316,7 +293,7 @@ BOOL GDKVideoOutputDevice::Redraw ()
   g_object_unref (tmp);
 
 
-  if (gconf_client_get_bool (client, "/apps/gnomemeeting/video_display/bilinear_filtering", NULL)) 
+  if (gconf_client_get_bool (client, VIDEO_DISPLAY_KEY "bilinear_filtering", NULL)) 
     interp_type = GDK_INTERP_BILINEAR;
 
 
@@ -383,7 +360,7 @@ BOOL GDKVideoOutputDevice::Redraw ()
 	  fullscreen = has_to_fs;
 	  has_to_switch_fs = true;
 	  gconf_client_set_bool (client, 
-				 "/apps/gnomemeeting/video_display/fullscreen", fullscreen, 0);
+				 VIDEO_DISPLAY_KEY "fullscreen", fullscreen, 0);
 	}
       }
 
@@ -580,6 +557,8 @@ BOOL GDKVideoOutputDevice::SetFrameData(
   if (!endFrame)
     return FALSE;
 
+  frameStore.SetSize (width * height * 3);
+  
   if (converter)
     converter->Convert (data, frameStore.GetPointer ());
   
