@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2001 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,16 +15,24 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                        toolbar.h  -  description
  *                        -------------------------
  *   begin                : Sat Dec 23 2000
- *   copyright            : (C) 2000-2001 by Damien Sandras
+ *   copyright            : (C) 2000-2003 by Damien Sandras
  *   description          : This file contains all the functions needed to
  *                          create the toolbar.
- *   email                : dsandras@seconix.com
  *
  */
 
@@ -32,14 +40,28 @@
 #ifndef _TOOLBAR_H_
 #define _TOOLBAR_H_
 
-#include <gnome.h>
 #include "common.h"
 
 
 /* DESCRIPTION  :  /
- * BEHAVIOR     :  Create the toolbars and add them to the main window.
+ * BEHAVIOR     :  Create the main toolbar and return a pointer to the newly
+ *                 created toolbar.
  * PRE          :  /
  */
-void gnomemeeting_init_toolbar ();
+GtkWidget *gnomemeeting_init_main_toolbar (void);
 
+
+/* DESCRIPTION  :  /
+ * BEHAVIOR     :  Create the left toolbar and return a pointer to the newly
+ *                 created toolbar.
+ * PRE          :  /
+ */
+GtkWidget *gnomemeeting_init_left_toolbar (void);
+
+
+/* DESCRIPTION  :  /
+ * BEHAVIOR     :  Updates the connect button.
+ * PRE          :  /
+ */
+void connect_button_update_pixmap (GtkToggleButton *, int);
 #endif
