@@ -41,115 +41,75 @@
 
 #include "videograbber.h"
 #include "gnomemeeting.h"
-#include "vfakeio.h"
 #include "misc.h"
-#include "menu.h"
+
 #include "dialog.h"
+#include "gconf_widgets_extensions.h"
 
 
 /* Declarations */
-extern GnomeMeeting *MyApp;
 extern GtkWidget *gm;
 
 
 /* The functions */
-GMVideoGrabber::GMVideoGrabber (PIntCondMutex *m,
-				BOOL start_grabbing,
+GMVideoGrabber::GMVideoGrabber (BOOL start_grabbing,
 				BOOL sync)
-  :PThread (1000, AutoDeleteThread)
+  :PThread (1000, NoAutoDeleteThread)
 {
   height = 0, width = 0;
   whiteness = 0, brightness = 0, colour = 0, contrast = 0;
 
-  gw = MyApp->GetMainWindow ();
-  pw = MyApp->GetPrefWindow ();
-  dw = MyApp->GetDruidWindow ();
-  
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  pw = GnomeMeeting::Process ()->GetPrefWindow ();
+  dw = GnomeMeeting::Process ()->GetDruidWindow ();
+
   /* Internal state */
-  is_running = 1;
-  has_to_close = 0;
-  has_to_reset = 0;
-  has_to_stop = 0;
+  stop = FALSE;
   is_grabbing = start_grabbing;
   synchronous = sync;
-  is_opened = 0;
+  is_opened = FALSE;
 
   /* Initialisation */
   encoding_device = NULL;
   channel = NULL;
   grabber = NULL;
   encoding_device = NULL;
-  client = gconf_client_get_default ();
-  color_format = NULL;
-  video_device = NULL;
   video_channel = 0;
   video_size = 0;
   video_format = PVideoDevice::Auto;
+  client = gconf_client_get_default ();
+
   UpdateConfig ();
 
   if (synchronous)
     VGOpen ();
 
-  number_of_instances_mutex = m;
-
-  ++(*number_of_instances_mutex);
-  
   /* Start the thread */
   this->Resume ();
+  thread_sync_point.Wait ();
 }
 
 
 GMVideoGrabber::~GMVideoGrabber ()
 {
-  PWaitAndSignal m(device_mutex);
+  stop = TRUE;
+  is_grabbing = 0;
+  device_mutex.Wait ();
 
-  g_free (color_format);
-  g_free (video_device);
-
-  --(*number_of_instances_mutex);
+  PWaitAndSignal m(quit_mutex);
 }
 
 
 void GMVideoGrabber::Main ()
 {
-  static PMutex mutex;
-  PWaitAndSignal m(mutex);
-
+  PWaitAndSignal m(quit_mutex);
   
-  /* We need those variables to facilitate the protection around mutexes */
-  int to_reset = 0, to_run = 0;
-
+  thread_sync_point.Signal ();
+ 
   if (!synchronous)
     VGOpen ();
 
-  var_mutex.Wait ();
-  to_run = is_running;
-  var_mutex.Signal ();
-  
-  while (to_run == 1) {
-
-    /* We protect variables that can be modified from several threads */
-    var_mutex.Wait ();
-    to_reset = has_to_reset;
-    to_run = is_running;
-    var_mutex.Signal ();
-
-    /* The user can ask several resets */
-    if (to_reset >= 1) {
-
-      VGClose ();
-      UpdateConfig ();
-      VGOpen ();
-      
-      var_mutex.Wait ();
-      has_to_reset--;
-      is_grabbing = 1;
-      
-      /* if we still need to reset more than one time, only reset one time */
-      if (has_to_reset >= 1) 
-	has_to_reset = 1;
-      var_mutex.Signal ();
-    }
+  while (!stop) {
 
     if (is_grabbing == 1) {
 
@@ -161,29 +121,18 @@ void GMVideoGrabber::Main ()
   }
 
   VGClose ();
-
-  gnomemeeting_threads_enter ();
-  gnomemeeting_init_main_window_logo (gw->main_video_image);
-  gnomemeeting_threads_leave ();
 }
 
 
 void GMVideoGrabber::UpdateConfig ()
 {
-  g_free (color_format);
-  g_free (video_device);
- 
   gnomemeeting_threads_enter ();
-  video_device =  gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_recorder", NULL);
+  video_recorder = gconf_get_string (VIDEO_DEVICES_KEY "input_device");
+  video_driver = gconf_get_string (VIDEO_DEVICES_KEY "plugin");
+  video_channel = gconf_get_int (VIDEO_DEVICES_KEY "channel");
+  video_size = gconf_get_int (VIDEO_DEVICES_KEY "size");
 
-  color_format = gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/color_format", NULL);
-
-  video_channel =  gconf_client_get_int (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_channel", NULL);
-
-  video_size =  gconf_client_get_int (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_size", NULL);
-
-
-  switch (gconf_client_get_int (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_format", NULL)) {
+  switch (gconf_get_int (VIDEO_DEVICES_KEY "format")) {
     
   case 0:
     video_format = PVideoDevice::PAL;
@@ -206,23 +155,6 @@ void GMVideoGrabber::UpdateConfig ()
     break;
   }
   gnomemeeting_threads_leave ();
-
-}
-
-
-void GMVideoGrabber::Close (void)
-{
-  var_mutex.Wait ();
-  is_running = 0;
-  var_mutex.Signal ();
-}
-
-
-void GMVideoGrabber::Reset (void)
-{
-  var_mutex.Wait ();
-  has_to_reset++;
-  var_mutex.Signal ();
 }
 
 
@@ -245,6 +177,18 @@ void GMVideoGrabber::StopGrabbing (void)
 }
 
 
+BOOL GMVideoGrabber::IsGrabbing (void)
+{
+  BOOL isg = FALSE;
+
+  var_mutex.Wait ();
+  isg = is_grabbing;
+  var_mutex.Signal ();
+
+  return isg;
+}
+
+
 GDKVideoOutputDevice *GMVideoGrabber::GetEncodingDevice (void)
 {
   return encoding_device;
@@ -312,42 +256,8 @@ void GMVideoGrabber::VGOpen (void)
   gchar *msg = NULL;
   gchar *video_image = NULL;
   int error_code = -1;  // No error
-  int opened;
 
-  PWaitAndSignal m(device_mutex);
-  
-  var_mutex.Wait ();
-  opened = is_opened;
-  var_mutex.Signal ();
-
-  if (!opened) {
-
-    /* Disable the video preview button while opening */
-    gnomemeeting_threads_enter ();
-    gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
-#ifndef DISABLE_GNOME
-    gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
-#endif
-    gnomemeeting_statusbar_flash (gw->statusbar, _("Opening Video device"));
-    gnomemeeting_log_insert (gw->history_text_view, _("Opening Video device"));
-    gnomemeeting_threads_leave ();
-    
-#ifdef TRY_1394AVC
-    if (video_device == "/dev/raw1394" ||
-       strncmp (video_device, "/dev/video1394", 14) == 0) {
-            grabber = new PVideoInput1394AvcDevice();
-       }
-    else
-#endif
-#ifdef TRY_1394DC
-    if (video_device == "/dev/raw1394" ||
-        strncmp (video_device, "/dev/video1394", 14) == 0)
-           grabber = new PVideoInput1394DcDevice();
-    else
-#endif
-      {
-	 grabber = new PVideoInputDevice();
-      }
+  if (!is_opened) {
 
     if (video_size == 0) { 
       
@@ -360,22 +270,32 @@ void GMVideoGrabber::VGOpen (void)
       width = GM_CIF_WIDTH; 
     }
     
-    grabber->SetPreferredColourFormat (color_format);
-
-    /* no error if Picture is choosen as video device */
-    if (!strcmp (video_device, _("Picture")))
+    if (video_recorder == _("No device found"))
       error_code = -2;
+    else {
 
-    if (error_code != -2) {
+      gnomemeeting_threads_enter ();
+      msg =
+	g_strdup_printf (_("Opening video device %s with driver %s"),
+			 (const char *) video_recorder,
+			 (const char *) video_driver);
+      gnomemeeting_log_insert (gw->history_text_view, msg);
+      g_free (msg);
+      gnomemeeting_threads_leave ();
+      
+      grabber = 
+	PVideoInputDevice::CreateOpenedDevice (video_driver,
+					       video_recorder,
+					       FALSE);
 
-      if (!grabber->Open (video_device, FALSE))
+      if (grabber == NULL)
 	error_code = 0;
-      else
+      else 
 	if (!grabber->SetVideoFormat(video_format))
 	  error_code = 2;
       else
-        if (!grabber->SetChannel(video_channel))
-          error_code = 2;
+	if (!grabber->SetChannel(video_channel))
+	  error_code = 2;
       else
 	if (!grabber->SetColourFormatConverter ("YUV420P"))
 	  error_code = 3;
@@ -387,34 +307,34 @@ void GMVideoGrabber::VGOpen (void)
 	  error_code = 5;
     }
 
-
     /* If no error */
     if (error_code == -1) {
 
       gnomemeeting_threads_enter ();
       msg = g_strdup_printf 
 	(_("Successfully opened video device %s, channel %d"), 
-	 video_device, video_channel);
+	 (const char *) video_recorder, video_channel);
       gnomemeeting_log_insert (gw->history_text_view, msg);
-      gnomemeeting_statusbar_flash (gw->statusbar, _("Video Device Opened"));
       g_free (msg);
       gnomemeeting_threads_leave ();
     }
     else {
-    
-      /* If we want to open the fake device for a real error, and not because
-	 the user chose the Picture device */
 
       if (error_code != -2) {
-
-	gnomemeeting_threads_enter ();
-	title_msg =
-	  g_strdup_printf (_("Error while opening video device %s"), video_device);
-			   
-	msg = g_strdup (_("The chosen Video Image will be transmitted during calls. If you didn't choose any image, then the default GnomeMeeting logo will be transmitted. Notice that you can always transmit a given image or the GnomeMeeting logo by choosing \"Picture\" as video device."));
-	gnomemeeting_statusbar_flash (gw->statusbar, _("Can't open the Video Device"));
 	
-	switch (error_code)	{
+	/* If we want to open the fake device for a real error, and not because
+	   the user chose the Picture device */
+	gnomemeeting_threads_enter ();
+	title_msg = g_strdup_printf (_("Error while opening video device %s"),
+				     (const char *) video_recorder);
+
+	/* Translators: Do not translate MovingLogo and StaticPicture */
+	msg = g_strdup (_("A moving GnomeMeeting logo will be transmitted during calls. Notice that you can always transmit a given image or the moving GnomeMeeting logo by choosing \"Picture\" as video plugin and \"MovingLogo\" or \"StaticPicture\" as device."));
+	gnomemeeting_statusbar_flash (gw->statusbar,
+				      _("Couldn't open the video device"));
+	gnomemeeting_log_insert (gw->history_text_view,
+				 _("Couldn't open the video device"));
+	switch (error_code) {
 	  
 	case 0:
 	  msg = g_strconcat (msg, "\n\n", _("There was an error while opening the device. Please check your permissions and make sure that the appropriate driver is loaded."), 
@@ -430,7 +350,7 @@ void GMVideoGrabber::VGOpen (void)
 	  break;
       
 	case 3:
-	  msg = g_strconcat (msg, "\n\n", g_strdup_printf(_("Your driver doesn't seem to support any of the colour formats supported by GnomeMeeting.\n Please check your kernel driver documentation in order to determine which Palette is supported. Set it as GnomeMeeting default with:\n gconftool --set \"/apps/gnomemeeting/devices/color_format\" YOURPALETTE --type string")), NULL);
+	  msg = g_strconcat (msg, "\n\n", g_strdup_printf(_("Your driver doesn't seem to support any of the colour formats supported by GnomeMeeting.\n Please check your kernel driver documentation in order to determine which Palette is supported.")), NULL);
 	  break;
       
 	case 4:
@@ -442,38 +362,46 @@ void GMVideoGrabber::VGOpen (void)
 	  break;
 	}
 
-	gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gm), gw->preview_button, title_msg, msg);
+	gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gm),
+					       gw->preview_button,
+					       title_msg,
+					       msg);
 	g_free (msg);
 	g_free (title_msg);
 
 	gnomemeeting_threads_leave ();
       }
-
+         
       /* delete the failed grabber and open the fake grabber, either
-       because there was an error, either because the user chose to do so */
-      delete grabber;
-      
-      gnomemeeting_threads_enter ();
-      video_image = gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_image", NULL);
-      gnomemeeting_threads_leave ();
+	 because there was an error, either because the user chose to do so */
+      if (grabber)
+	delete grabber;
 
-      grabber = new GMH323FakeVideoInputDevice (video_image);
-      grabber->SetColourFormatConverter ("YUV420P");
-      grabber->SetVideoFormat (PVideoDevice::PAL);
-      grabber->SetChannel (1);    
-      grabber->SetFrameRate (6);
-      grabber->SetFrameSizeConverter (width, height, FALSE);
 
       gnomemeeting_threads_enter ();
-#ifndef DISABLE_GNOME
-      gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), true);
-#endif
+      video_image = gconf_get_string (VIDEO_DEVICES_KEY "image");
       gnomemeeting_threads_leave ();
+    
+      grabber =
+	PVideoInputDevice::CreateOpenedDevice ("Picture",
+					       "MovingLogo",
+					       FALSE);
+      if (grabber) {
+	
+	grabber->SetColourFormatConverter ("YUV420P");
+	grabber->SetVideoFormat (PVideoDevice::PAL);
+	grabber->SetChannel (1);    
+	grabber->SetFrameRate (6);
+	grabber->SetFrameSizeConverter (width, height, FALSE);
+
+	gnomemeeting_threads_enter ();
+	gnomemeeting_log_insert (gw->history_text_view, _("Successfully opened video device with the \"Picture\" video plugin"));
+	gnomemeeting_threads_leave ();
+      }
       
       g_free (video_image);
     }
-
-    
+      
     grabber->Start ();
 
     var_mutex.Wait ();
@@ -486,7 +414,7 @@ void GMVideoGrabber::VGOpen (void)
     channel->AttachVideoPlayer (encoding_device);
 
     var_mutex.Wait ();
-    is_opened = 1;
+    is_opened = TRUE;
     var_mutex.Signal ();
   
     encoding_device->SetFrameSize (width, height);  
@@ -494,84 +422,50 @@ void GMVideoGrabber::VGOpen (void)
     /* Setup the video settings */
     GetParameters (&whiteness, &brightness, &colour, &contrast);
     gnomemeeting_threads_enter ();
-    gtk_adjustment_set_value (GTK_ADJUSTMENT (gw->adj_brightness),
-			      brightness);
-    gtk_adjustment_set_value (GTK_ADJUSTMENT (gw->adj_whiteness),
-			      whiteness);
-    gtk_adjustment_set_value (GTK_ADJUSTMENT (gw->adj_colour),
-			      colour);
-    gtk_adjustment_set_value (GTK_ADJUSTMENT (gw->adj_contrast),
-			      contrast);
-    
+    GTK_ADJUSTMENT (gw->adj_brightness)->value = brightness;
+    GTK_ADJUSTMENT (gw->adj_whiteness)->value = whiteness;
+    GTK_ADJUSTMENT (gw->adj_colour)->value = colour;
+    GTK_ADJUSTMENT (gw->adj_contrast)->value = contrast;
+    gtk_widget_queue_draw (GTK_WIDGET (gw->video_settings_frame));
+
     /* Enable the video settings frame */
     gtk_widget_set_sensitive (GTK_WIDGET (gw->video_settings_frame),
 			      TRUE);
     gnomemeeting_threads_leave ();
 
     /* Enable the video preview button if not in a call */
-    if (MyApp->Endpoint ()->GetCallingState () == 0) {
+    if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () == GMH323EndPoint::Standby) {
 
       gnomemeeting_threads_enter ();      
-      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), true);
+      gnomemeeting_menu_update_sensitivity (TRUE, FALSE, TRUE);
       gnomemeeting_threads_leave ();
     }
-
-
-    /* Enable zoom and fullscreen settings in the view menu */
-    gnomemeeting_threads_enter ();
-    gnomemeeting_zoom_submenu_set_sensitive (TRUE);
-
-#ifdef HAS_SDL
-    gnomemeeting_fullscreen_option_set_sensitive (TRUE);
-#endif
-
-    gnomemeeting_threads_leave ();
-  }  
+  }
 }
   
 
 void GMVideoGrabber::VGClose ()
 {
-  int opened;
-
-  PWaitAndSignal m(device_mutex);  
-
-  var_mutex.Wait ();
-  opened = is_opened;
-  var_mutex.Signal ();
-
-  if (opened) {
+  if (is_opened) {
 
     var_mutex.Wait ();
     is_grabbing = 0;
     var_mutex.Signal ();
 
-    /* Disable the video preview button while closing */
-    gnomemeeting_threads_enter ();
-    gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
-#ifndef DISABLE_GNOME
-    gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
-#endif
-    gnomemeeting_threads_leave ();
-    
-    if (channel) 
+
+    if (channel)
       delete (channel);
 
-    var_mutex.Wait ();
-    has_to_close = 0;
-    is_opened = 0;
-    var_mutex.Signal ();
-  
+    
     /* Enable video preview button */
     gnomemeeting_threads_enter ();
   
-    /* Enable the video preview button  */
-    gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), TRUE);
-#ifndef DISABLE_GNOME
-    gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), TRUE);
-#endif
-    
-    gnomemeeting_statusbar_flash (gw->statusbar, _("Video Device Closed"));
+
+    if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () == GMH323EndPoint::Standby && !gconf_get_bool (VIDEO_DEVICES_KEY "enable_preview")) {
+      
+      gnomemeeting_menu_update_sensitivity (TRUE, FALSE, FALSE);
+      gnomemeeting_init_main_window_logo (gw->main_video_image);
+    }
     
     /* Disable the video settings frame */
     gtk_widget_set_sensitive (GTK_WIDGET (gw->video_settings_frame),
@@ -580,21 +474,11 @@ void GMVideoGrabber::VGClose ()
     gnomemeeting_threads_leave ();
     
     
-    /* Disable the zoom and fullscreen options */
-    gnomemeeting_threads_enter ();
-    gnomemeeting_zoom_submenu_set_sensitive (FALSE);
-
-#ifdef HAS_SDL
-    gnomemeeting_fullscreen_option_set_sensitive (FALSE);
-#endif
-
-    gnomemeeting_threads_leave ();
-
-
     /* Initialisation */
     grabber = NULL;
     var_mutex.Wait ();
     channel = NULL;
+    is_opened = FALSE;
     encoding_device = NULL;
     var_mutex.Signal ();
   }
@@ -606,12 +490,21 @@ void GMVideoGrabber::VGClose ()
 
 
 /* The video tester class */
-GMVideoTester::GMVideoTester ()
+GMVideoTester::GMVideoTester (gchar *m,
+			      gchar *r)
   :PThread (1000, AutoDeleteThread)
 {
 #ifndef DISABLE_GNOME
+  if (m)
+    video_manager = PString (m);
+  if (r)
+    video_recorder = PString (r);
 
+  test_dialog = NULL;
+  test_label = NULL;
+  
   this->Resume ();
+  thread_sync_point.Wait ();
 #endif
 }
 
@@ -619,79 +512,96 @@ GMVideoTester::GMVideoTester ()
 GMVideoTester::~GMVideoTester ()
 {
 #ifndef DISABLE_GNOME
-  quit_mutex.Wait ();
-
-  quit_mutex.Signal ();
- #endif
+  PWaitAndSignal m(quit_mutex);
+#endif
 }
 
 
 void GMVideoTester::Main ()
 {
 #ifndef DISABLE_GNOME
-  quit_mutex.Wait ();
-
   GmWindow *gw = NULL;
   GmDruidWindow *dw = NULL;
+
   PVideoInputDevice *grabber = NULL;
   
   int height = GM_QCIF_HEIGHT; 
   int width = GM_QCIF_WIDTH; 
   int error_code = -1;
   int cpt = 0;
-  gdouble per = 0.0;
 
   gchar *msg = NULL;
-  gchar *video_device = NULL;
+  gchar *tmp = NULL;
 
-  gnomemeeting_threads_enter ();
-  GConfClient *client = gconf_client_get_default ();
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  dw = GnomeMeeting::Process ()->GetDruidWindow ();
 
-  video_device =  gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_recorder", NULL);
+  PWaitAndSignal m(quit_mutex);
+  thread_sync_point.Signal ();
 
-  gw = MyApp->GetMainWindow ();
-  dw = MyApp->GetDruidWindow ();
-  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
-  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), "");
-  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), 0.0);
-  gnomemeeting_threads_leave ();
-
-#ifdef TRY_1394AVC
-    if (video_device == "/dev/raw1394" ||
-       strncmp (video_device, "/dev/video1394", 14) == 0) {
-            grabber = new PVideoInput1394AvcDevice();
-       }
-    else
-#endif
-#ifdef TRY_1394DC
-    if (video_device == "/dev/raw1394" ||
-        strncmp (video_device, "/dev/video1394", 14) == 0)
-           grabber = new PVideoInput1394DcDevice();
-    else
-#endif
-      {
-	 grabber = new PVideoInputDevice();
-      }
+  if (video_recorder.IsEmpty ()
+      || video_manager.IsEmpty ()
+      || video_recorder == PString (_("No device found"))
+      || video_recorder == PString (_("Picture")))
+    return;
   
-  while (cpt <= 3) {
+  gdk_threads_enter ();
+  test_dialog =
+    gtk_dialog_new_with_buttons ("Video test running",
+				 GTK_WINDOW (gw->druid_window),
+				 (enum GtkDialogFlags) (GTK_DIALOG_MODAL),
+				 GTK_STOCK_OK,
+				 GTK_RESPONSE_ACCEPT,
+				 NULL);
+  msg = 
+    g_strdup_printf (_("GnomeMeeting is now testing the %s video device. If you experience machine crashes, then report a bug to the video driver author."), (const char *) video_recorder);
+  test_label = gtk_label_new (msg);
+  gtk_label_set_line_wrap (GTK_LABEL (test_label), true);
+  g_free (msg);
 
-    gdk_threads_enter ();
-    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), per);
-    gtk_widget_queue_draw (GTK_WIDGET (dw->progress));
-    gdk_threads_leave ();
-    
-    if (strcmp (video_device, _("Picture"))) {
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (test_dialog)->vbox), test_label,
+		      FALSE, FALSE, 2);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (test_dialog)->vbox), 
+		      gtk_hseparator_new (), FALSE, FALSE, 2);
+
+  test_label = gtk_label_new (NULL);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (test_dialog)->vbox), 
+		      test_label, FALSE, FALSE, 2);
+
+  g_signal_connect (G_OBJECT (test_dialog), "delete-event",
+		    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+
+  gtk_window_set_transient_for (GTK_WINDOW (test_dialog),
+				GTK_WINDOW (gw->druid_window));
+  gtk_widget_show_all (test_dialog);
+  gdk_threads_leave ();
+
+  
+  while (cpt < 6 && error_code == -1) {
+
+    if (!video_recorder.IsEmpty ()
+	&& !video_manager.IsEmpty ()) {
+
+      error_code = -1;
       
-      if (!grabber->Open (video_device, FALSE))
+      grabber = 
+	PVideoInputDevice::CreateOpenedDevice (video_manager,
+					       video_recorder,
+					       FALSE);
+
+      if (!grabber)
 	error_code = 0;
       else
-	if (!grabber->SetVideoChannelFormat (0,  PVideoDevice::Auto))
+	if (!grabber->SetVideoFormat (PVideoDevice::Auto))
 	  error_code = 2;
+      else
+        if (!grabber->SetChannel (0))
+          error_code = 2;
       else
 	if (!grabber->SetColourFormatConverter ("YUV420P"))
 	  error_code = 3;
       else
-	if (!grabber->SetFrameRate (10))
+	if (!grabber->SetFrameRate (30))
 	  error_code = 4;
       else
 	if (!grabber->SetFrameSizeConverter (width, height, FALSE))
@@ -699,28 +609,35 @@ void GMVideoTester::Main ()
       else
 	grabber->Close ();
 
-      if (error_code != -1)
-	break;
+      if (grabber)
+	delete (grabber);
     }
-    
-    msg = g_strdup_printf (_("Test %d done"), cpt);
+
+
+    if (error_code == -1) 
+      msg = g_strdup_printf (_("Test %d done"), cpt);
+    else
+      msg = g_strdup_printf (_("Test %d failed"), cpt);
+
+    tmp = g_strdup_printf ("<b>%s</b>", msg);
     gdk_threads_enter ();
-    gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), msg);
+    gtk_label_set_markup (GTK_LABEL (test_label), tmp);
     gdk_threads_leave ();
     g_free (msg);
+    g_free (tmp);
 
-    per = cpt * 0.33;
     cpt++;
     PThread::Current () ->Sleep (100);
   }
 
-
+  
   if (error_code != - 1) {
     
     switch (error_code)	{
 	  
     case 0:
-      msg = g_strdup_printf (_("Error while opening %s."), video_device);
+      msg = g_strdup_printf (_("Error while opening %s."),
+			     (const char *) video_recorder);
       break;
       
     case 1:
@@ -732,7 +649,7 @@ void GMVideoTester::Main ()
       break;
       
     case 3:
-      msg = g_strdup_printf (_("Your driver doesn't support the YUV420P color format"));
+      msg = g_strdup_printf (_("Your driver doesn't support any of the color formats tried by GnomeMeeting"));
       break;
       
     case 4:
@@ -743,25 +660,22 @@ void GMVideoTester::Main ()
       msg = g_strdup_printf (_("Error with the frame size."));
       break;
     }
-  }  
-  else
-    msg = g_strdup (_("Tests OK!"));
+
+    gdk_threads_enter ();
+    gnomemeeting_error_dialog (GTK_WINDOW (gw->druid_window),
+			       _("Failed to open the device"),
+			       msg);
+    gdk_threads_leave ();
+    
+    g_free (msg);
+  }
 
   gdk_threads_enter ();
-  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), msg);
-  gdk_threads_leave ();
-  g_free (msg);
-
-  delete (grabber);
-
-  gdk_threads_enter ();
-  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), 1.0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dw->video_test_button),
 				FALSE);
-  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), TRUE);
+  if (test_dialog)
+    gtk_widget_destroy (test_dialog);
   gdk_threads_leave ();
-
-  quit_mutex.Signal ();
 #endif
 }
 
